@@ -22,11 +22,13 @@ Net::CardDAVTalk - A library for talking to CardDAV servers
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
+
+our $BATCHSIZE = 100;
 
 
 =head1 SYNOPSIS
@@ -458,7 +460,6 @@ contacts in scalar context again)
 
 sub GetContacts {
   my ($Self, $Path, $Props) = @_;
-  $Props //= [];
 
   my $data = $Self->GetContactLinks($Path);
   my @AllUrls = sort keys %$data;
@@ -516,41 +517,44 @@ also fetching \$Props on top of the address-data and getetag.
 
 sub GetContactsMulti {
   my ($Self, $Path, $Urls, $Props) = @_;
-  my (@Contacts, @Errors, %ETags);
+  $Props //= [];
+  my (@Contacts, @Errors, %Links);
 
-  my $Response = $Self->Request(
-    'REPORT',
-    "$Path/",
-    x('C:addressbook-multiget', $Self->NS(),
-      x('D:prop',
-        x('D:getetag'),
-        x('C:address-data'),
-        map { x(join ":", @$_) } @$Props,
+  while (my @urls = splice(@$Urls, 0, $BATCHSIZE)) {
+    my $Response = $Self->Request(
+      'REPORT',
+      "$Path/",
+      x('C:addressbook-multiget', $Self->NS(),
+        x('D:prop',
+          x('D:getetag'),
+          x('C:address-data'),
+          map { x(join ":", @$_) } @$Props,
+        ),
+        map { x('D:href', $_) } @urls,
       ),
-      map { x('D:href', $_) } @$Urls,
-    ),
-    Depth => '0',
-  );
+      Depth => '0',
+    );
 
-  my $NS_C = $Self->ns('C');
-  my $NS_D = $Self->ns('D');
-  foreach my $Response (@{$Response->{"{$NS_D}response"} || []}) {
-    my $href = $Response->{"{$NS_D}href"}{content};
-    next unless $href;
-    foreach my $Propstat (@{$Response->{"{$NS_D}propstat"} || []}) {
-      my $etag = $Propstat->{"{$NS_D}prop"}{"{$NS_D}getetag"}{content} || '';
-      my $VCard = eval { $Self->_ParseReportData($Response, $Propstat, $Props) } || do {
-        push @Errors, $@ if $@;
-        next;
-      };
+    my $NS_C = $Self->ns('C');
+    my $NS_D = $Self->ns('D');
+    foreach my $Response (@{$Response->{"{$NS_D}response"} || []}) {
+      my $href = $Response->{"{$NS_D}href"}{content};
+      next unless $href;
+      foreach my $Propstat (@{$Response->{"{$NS_D}propstat"} || []}) {
+        my $etag = $Propstat->{"{$NS_D}prop"}{"{$NS_D}getetag"}{content} || '';
+        my $VCard = eval { $Self->_ParseReportData($Response, $Propstat, $Props) } || do {
+          push @Errors, $@ if $@;
+          next;
+        };
 
-      push @Contacts, $VCard;
+        push @Contacts, $VCard;
 
-      $ETags{$href} = $etag;
+        $Links{$href} = $etag;
+      }
     }
   }
 
-  return wantarray ? (\@Contacts, \@Errors, \%ETags) : \@Contacts;
+  return wantarray ? (\@Contacts, \@Errors, \%Links) : \@Contacts;
 }
 
 =head2 $self->SyncContacts($Path, $Props, %Args)
@@ -566,14 +570,14 @@ list context.
 sub SyncContacts {
   my ($Self, $Path, $Props, %Args) = @_;
 
-  my ($Links, $Removed, $Errors, $SyncToken) = $Self->SyncContactLinks($Path, %Args);
+  my ($Added, $Removed, $Errors, $SyncToken) = $Self->SyncContactLinks($Path, %Args);
 
-  my @AllUrls = sort keys %$Links;
+  my @AllUrls = sort keys %$Added;
 
-  my ($Contacts, $ThisErrors, $ETags) = $Self->GetContactsMulti($Path, \@AllUrls, $Props);
+  my ($Contacts, $ThisErrors, $Links) = $Self->GetContactsMulti($Path, \@AllUrls, $Props);
   push @$Errors, @$ThisErrors;
 
-  return wantarray ? ($Contacts, $Removed, $Errors, $SyncToken, $ETags) : $Contacts;
+  return wantarray ? ($Contacts, $Removed, $Errors, $SyncToken, $Links) : $Contacts;
 }
 
 =head2 $self->SyncContactLinks($Path, %Args)
@@ -699,6 +703,7 @@ sub _ParseReportData {
   return unless $VCard;
 
   $VCard->{CPath} = $CPath;
+  $VCard->{href} = $HRef;
 
   my %Props;
   for (@$Props) {
